/* 
 * Copyright (C) 2012 Changwoo Ryu
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __PROPS_DATA_H__
#define __PROPS_DATA_H__

#include <gsf/gsf-doc-meta-data.h>

GsfDocMetaData *props_data_read(const char *filename, GError **error);

#endif /* __PROPS_DATA_H__ */
